{
    encoders => {
        json => {
            indented => {
                indent => 1,
            },
        },
    },

    modules_init => {
        JSON => {
            indent => 0,
            space_before => 0,
        },
    },
}

